package mc600.boardgames;

import java.util.Vector;

public class Root {

	protected Vector free = new Vector();
	protected Vector occupied = new Vector();
	protected BoardPosition origin;

	public Root(BoardPosition origin) {
		this.origin = origin;
		BoardPosition[] neighbours = origin.neighbours();
		for (int i = 0; i < neighbours.length; i++) {
			free.add(neighbours[i]);
		}
		occupied.add(origin);
	}

	/*
	 * Retorna as posicoes livres de uma raiz
	 */
	public BoardPosition[] getFreePositions() {
		return (BoardPosition[]) free.toArray(new BoardPosition[0]);
	}

	/*
	 * Retorna o ponto de origem da raiz, isto e, a primeira posicao ocuopada
	 * por ela.
	 */
	public BoardPosition getOrigin() {
		return origin;
	}

	/*
	 * Retorna as posicoes ocupadas de uma raiz
	 */
	public BoardPosition[] occupied() {
		return (BoardPosition[]) occupied.toArray(new BoardPosition[0]);
	}

	/*
	 *  Retorna TRUE se a raiz nao pode mais ser extendida.
	 */
	public boolean isDead() {
		return free.isEmpty();
	}

	/*
	 *  Retorna um vetor de vizinhos de uma colecao de posicoes.
	 */
	public BoardPosition[] getNeighbourhood() {
		Vector neigbourhood = new Vector();
		BoardPosition[] aux = null;
		for (int i = 0; i < occupied.size(); i++) {
			aux = ((BoardPosition) occupied.elementAt(i)).neighbours();
			for (int j = 0; j < aux.length; j++) {
				if ((!isOccupied(aux[j]))
					&& (!contains(neigbourhood, aux[j]))) {
					neigbourhood.add(aux[j]);
				}
			}
		}
		return (BoardPosition[]) neigbourhood.toArray(new BoardPosition[0]);
	}

	/*
	 * Remove um conjunto de posicoes livres da raiz
	 */
	protected void removeFree(BoardPosition[] toRemove) {
		for (int i = 0; i < toRemove.length; i++) {
			free.remove(toRemove[i]);
		}
	}

	/*
	 * Remove todas as posicoes livres da raiz
	 */
	protected void removeAllFree() {
		free = new Vector();
	}

	/*
	 * 
	 * Remove uma posicao livre da raiz
	 */
	protected void removeFree(BoardPosition toRemove) {
		BoardPosition p = null;
		for (int i = 0; i < free.size(); i++) {
			p = (BoardPosition) free.elementAt(i);
			if ((p.getY() == toRemove.getY())
				&& (p.getX() == toRemove.getX())) {
				free.remove(i);
				return;
			}
		}
		return;
	}

	/*
	 * Adiciona um conjunto de posicoes livres na raiz.
	 */
	protected void addFree(BoardPosition[] toAdd) {
		for (int i = 0; i < toAdd.length; i++) {
			if (!isFree(toAdd[i])) {
				BoardPosition[] neighbours = toAdd[i].neighbours();
				int neighboursInRoot = 0;
				for (int j = 0; j < neighbours.length; j++) {
					if (isOccupied(neighbours[j]))
						neighboursInRoot++;
				}
				if (neighboursInRoot == 1) {
					free.add(toAdd[i]);
				}
			}
		}
	}

	/*
	 * Troca o conjunto de posicoes livres da raiz por um novo conjunto fornecido.
	 */
	protected void renewFree(BoardPosition[] newFree) {
		free.clear();
		for (int i = 0; i < newFree.length; i++) {
			free.add(newFree[i]);
		}
	}

	/*
	 * Adicionia uma nova posicao ocupada na raiz.
	 */
	protected void addOccupied(BoardPosition bp) throws InvalidMoveException {
		BoardPosition[] neighbours = bp.neighbours();
		int neighboursInRoot = 0;
		for (int i = 0; i < neighbours.length; i++) {
			if (isOccupied(neighbours[i])) {
				neighboursInRoot++;
			}
		}
		if (neighboursInRoot != 1) {
			throw new InvalidMoveException("Esta posicao nao pode ser adicionada a esta raiz!");
		}
		if (!isFree(bp)) {
			throw new InvalidMoveException("Esta posicao nao pode ser adicionada a esta raiz!");
		}
		occupied.add(bp);
		removeFree(bp);
	}

	/*
	 *  Retorna TRUE se a posicao em questa e ocupada pela raiz
	 */
	public boolean isOccupied(BoardPosition p) {
		return contains(occupied, p);
	}

    /*
     *  Retorna TRUE se a posicao em questa e uma das posicoes livres da raiz.
     */
	public boolean isFree(BoardPosition p) {
		return contains(free, p);
	}

    /*
     * Retorna TRUE se o vetor de posicoes "v" contem a posicao "p"
     */
	private boolean contains(Vector v, BoardPosition p) {
		BoardPosition t = null;
		boolean toReturn = false;
		for (int i = 0; i < v.size(); i++) {
			t = (BoardPosition) v.elementAt(i);
			if ((t.getX() == p.getX()) && (t.getY() == p.getY())) {
				return true;
			}
		}
		return false;
	}

	public void print() {
		System.out.println(
			"<< ROOT " + origin.getX() + "," + origin.getY() + ">>");
		System.out.print("Ocupadas : ");
		printArray(occupied);
		System.out.print("Livres : ");
		printArray(free);
	}

	private void printArray(Vector bp) {
		System.out.print("[ ");
		for (int i = 0; i < bp.size(); i++) {
			System.out.print(
				"("
					+ ((BoardPosition) bp.elementAt(i)).getX()
					+ ","
					+ ((BoardPosition) bp.elementAt(i)).getY()
					+ ")");
		}
		System.out.println(" ]");
	}

}
